<?php
/*
Plugin Name: Last.fm Artists
Plugin URI: http://finalstar.net/lastfm
Description: Last.fm Artists shows images of your top listened to artists. <a href="plugins.php?page=lastfmartists/lastfm.php">Confiure Last.fm Artists</a>.
Version: 1.0.5
Author: Gary Fenstamaker 
Author URI: http://finalstar.net
Copyright 2008  Gary Fenstamaker (email : garyf@finalstar.net)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

Acknowledgment to Tim Eckel for the fastimagecopyresampled function
*/

function fastimagecopyresampled (&$dst_image, $src_image, $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h, $quality = 3) {
  // Plug-and-Play fastimagecopyresampled function replaces much slower imagecopyresampled.
  // Just include this function and change all "imagecopyresampled" references to "fastimagecopyresampled".
  // Typically from 30 to 60 times faster when reducing high resolution images down to thumbnail size using the default quality setting.
  // Author: Tim Eckel - Date: 09/07/07 - Version: 1.1 - Project: FreeRingers.net - Freely distributable - These comments must remain.
  //
  // Optional "quality" parameter (defaults is 3). Fractional values are allowed, for example 1.5. Must be greater than zero.
  // Between 0 and 1 = Fast, but mosaic results, closer to 0 increases the mosaic effect.
  // 1 = Up to 350 times faster. Poor results, looks very similar to imagecopyresized.
  // 2 = Up to 95 times faster.  Images appear a little sharp, some prefer this over a quality of 3.
  // 3 = Up to 60 times faster.  Will give high quality smooth results very close to imagecopyresampled, just faster.
  // 4 = Up to 25 times faster.  Almost identical to imagecopyresampled for most images.
  // 5 = No speedup. Just uses imagecopyresampled, no advantage over imagecopyresampled.

  if (empty($src_image) || empty($dst_image) || $quality <= 0) { return false; }
  if ($quality < 5 && (($dst_w * $quality) < $src_w || ($dst_h * $quality) < $src_h)) {
    $temp = imagecreatetruecolor ($dst_w * $quality + 1, $dst_h * $quality + 1);
    imagecopyresized ($temp, $src_image, 0, 0, $src_x, $src_y, $dst_w * $quality + 1, $dst_h * $quality + 1, $src_w, $src_h);
    imagecopyresampled ($dst_image, $temp, $dst_x, $dst_y, 0, 0, $dst_w, $dst_h, $dst_w * $quality, $dst_h * $quality);
    imagedestroy ($temp);
  } else imagecopyresampled ($dst_image, $src_image, $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h);
  return true;
}

function lastfm_checkFileCache($url) {
	if (file_exists($url)) {
		if (($_SERVER['REQUEST_TIME'] - filemtime($url)) > '86400') {
			// 86400 seconds = 1 day
			return false;
		}else{
			return true;
		}
	}else{
		return false;
	}
}

function lastfm_checkImageCache($url) {
	if (file_exists($url)) {
		if (($_SERVER['REQUEST_TIME'] - filemtime($url)) > '604800') {
			// 604800 seconds = 1 week
			return false;
		}else{
			return true;
		}
	}else{
		return false;
	}
}

// Gets the artist image for weekly feeds
function lastfm_image($name) {
	$name = urlencode($name);
	$url = 'http://ws.audioscrobbler.com/2.0/?method=artist.getinfo&api_key=ffd6c3e445e45d0d4bc124184853b772&artist=' . $name;
	
	$xml = simplexml_load_file($url);
	return $xml->artist->image[2];
}

function lastfm_createCache($filename, $contents) {
	$file = fopen($filename, 'w');
    fwrite($file, $contents);
    fclose($file);
    return $filename;
}

function lastfm_imageResize($image, $size, $filename, $mime, $width, $height) {
	// Creates the filename for either png or jpeg or gif files
    switch ($mime) {
    	case 'jpg':
        case 'jpeg':
            $src_img = imagecreatefromjpeg($image);
            break;
        case 'png':
            $src_img = imagecreatefrompng($image);
            break;
        case 'gif':
            $src_img = imagecreatefromgif($image);
            break;
    }
	// Finds the ratio of width to height
    $ratio = $width / $height;
    // Creates the new dimensions
    $new_width = ($ratio < 1) ? round($size * $ratio) : $size;
    $new_height = ($ratio > 1) ? round($size / $ratio) : $size;
    // Finds the differences between the new dimensions and the size desired
    $x = $size - $new_width;
    $y = $size - $new_height;
    // Adds the differences to the dimensions so they can be cut into a square
    $new_width = $new_width + $y + $x;
    $new_height = $new_height + $y + $x;
    
    // Cuts the image into a square
    $dst_img = imagecreatetruecolor($size, $size);
    fastimagecopyresampled($dst_img, $src_img, 0, 0, ($y/2), 0, $new_width, $new_height, $width, $height);
    // Saves the images
    switch ($mime) {
    	case 'jpg':
        case 'jpeg':
            imagejpeg($dst_img, $filename, 100);
            break;
        case 'png':
            imagepng($dst_img, $filename, 9);
            break;
        case 'gif':
            imagegif($dst_img, $filename);
            break;
    }
    // Detroys temp files
    imagedestroy($src_img);
    imagedestroy($dst_img);
	$filename = '/' . $filename;
    return $filename;
}

function lastfm_main($args = NULL) {
	$cacheDir = $_SERVER['DOCUMENT_ROOT'] . "/wp-content/lastfmcache/";
	if (file_exists($cacheDir) === false) {
		if (mkdir($cacheDir, 0755) === false) {
			return "Cache Directory could not be created.  Please create cache directory at '/wp-content/lastfmcache/' with CHMOD of 0755.";
		}
	}
	
	$opt = get_option('lastfmArtistsOptions');
	if (isset($args)) {
		foreach ($args as $key => $option) {
			$opt[$key] = $option;
		}
	}
	
	if ($opt['time'] == 'weekly') {
		$opt['feed'] = 'http://ws.audioscrobbler.com/2.0/?method=user.getweeklyartistchart&user=' . $opt['user'] . "&api_key=ffd6c3e445e45d0d4bc124184853b772";
	} else {
		$opt['feed'] = 'http://ws.audioscrobbler.com/2.0/?method=user.gettopartists&user=' . $opt['user'] . '&api_key=ffd6c3e445e45d0d4bc124184853b772';
	}
	
	$specialChars = array(' ', '+', '?','!','@','#','$','%','^','&','*','(',')','{','}','<','>','[',']','|','"',"'",';',',','.','/','~','`','=', ':');
	$displayDir = "/wp-content/lastfmcache/";
		
	$cachefile = $cacheDir . $opt['time'] . $opt['number'] . $opt['size'] . '.html';
	$displayfile = $displayDir . $opt['time'] . $opt['number'] . $opt['size'] . '.html';
	
	if (lastfm_checkFileCache($cachefile)) {
		$result = file_get_contents($cachefile);
		return $result;
	}
	
	// Checks if the feed is down.  If it is, then display the cache no matter what.
	$headers = get_headers($opt['feed']);
	if (file_exists($cachefile) && $headers['0'] != 'HTTP/1.0 200 OK') {
		$result = file_get_contents($cachefile);
		return $result;
	}elseif (!file_exists($cachefile) && $headers['0'] != 'HTTP/1.0 200 OK') {
		return false;
	}
	
	// Loads up some feeds
	$xml = simplexml_load_file($opt['feed']);
	$contents = '<ul class="lastfm_list">';
	$limit = $opt['number'];
	
	if ($opt['time'] == 'weekly') {				
		for ($counter = 0; $counter < $limit; $counter++) {
			// If the limit goes beyond five plus the original number, then stop
			if ($limit - $opt['number'] > $opt['numebr'] + 5 ){
				break;
			}
			$name = $xml->weeklyartistchart->artist[$counter]->name;
			$url = $xml->weeklyartistchart->artist[$counter]->url;
						
			$sin_name = str_replace($specialChars, '', $name);
			$filename = $cacheDir . $sin_name . $opt['size'];
			$displayname = $displayDir . $sin_name . $opt['size'];
			
			// Checks if there is anykind of cached image
			foreach (array('jpeg', 'png', 'gif', 'jpg') as $ext) {
				$image_url = $filename .'.'. $ext;
				$display_url = $displayname .'.'. $ext;
				if (lastfm_checkImageCache($image_url)) {
					$image_url = '/' . $image_url;
					$contents .= "<li class='lastfm_item'><a href=\"$url\"><img src=\"$display_url\" alt=\"$name\" title=\"$name\" class=\"lastfm_image\" /></a></li>";
					continue 2;
				}
			}
			
			$image = lastfm_image($name);
			// If there is no image, this skips over it and adds on to the limit
			if ($image == '') {
				$counter++;
				$limit++;
				continue;
			}
			
			$imageinfo = getimagesize($image);
			$mime = str_split($imageinfo['mime'], 6);
			$filename .= '.' . $mime[1];
			$displayname .= '.' . $mime[1];
			// Appends the mime type to the end of the filename to act as a file extentsion
			// Calls the imageResize function. If it can't resize, it throws an error stopping the exectution of the loop
			if (lastfm_imageResize($image, $opt['size'], $filename, $mime[1], $imageinfo[0], $imageinfo[1])) {
				$contents .= "<li class='lastfm_item'><a href=\"$url\"><img src=\"$displayname\" alt=\"$name\" title=\"$name\" class=\"lastfm_image\" /></a></li>";
			} else {
				return false;
			}
		}		
	}else{
		for ($counter = 0; $counter < $limit; $counter++) {
			// If the limit goes beyond five plus the original number, then stop
			if ($limit - $opt['number'] > $opt['numebr'] + 5 ){
				break;
			}
			$name = $xml->topartists->artist[$counter]->name;
			$url = $xml->topartists->artist[$counter]->url;
			$image = $xml->topartists->artist[$counter]->image[2];
			
			$sin_name = str_replace($specialChars, '', $name);
			$filename = $cacheDir . $sin_name . $opt['size'];
			$displayname = $displayDir . $sin_name . $opt['size'];
			
			// Checks if there is anykind of cached image
			foreach (array('jpeg', 'png', 'gif', 'jpg') as $ext) {
				$image_url = $filename .'.'. $ext;
				$display_url = $displayname .'.'. $ext;
				if (lastfm_checkImageCache($image_url)) {
					$image_url = '/' . $image_url;
					$contents .= "<li class='lastfm_item'><a href=\"$url\"><img src=\"$display_url\" alt=\"$name\" title=\"$name\" class=\"lastfm_image\" /></a></li>";
					continue 2;
				}
			}
			
			// If there is no image, this skips over it and adds on to the limit
			if ($image == '') {
				$counter++;
				$limit++;
				continue;
			}
			
			$imageinfo = getimagesize($image);
			$mime = str_split($imageinfo['mime'], 6);
			$filename .= '.' . $mime[1];
			$displayname .= '.' . $mime[1];
			// Appends the mime type to the end of the filename to act as a file extentsion
			// Calls the imageResize function. If it can't resize, it throws an error stopping the exectution of the loop
			if (lastfm_imageResize($image, $opt['size'], $filename, $mime[1], $imageinfo[0], $imageinfo[1])) {
				$contents .= "<li class='lastfm_item'><a href=\"$url\"><img src=\"$displayname\" alt=\"$name\" title=\"$name\" class=\"lastfm_image\" /></a></li>";
			} else {
				return false;
			}
			
		}
		
	}
	
	// If no images were added to $contents
	if ($contents == '<ul class="lastfm_list">') {
		return false;
	}
	$contents .= '</ul>';
	lastfm_createCache($cachefile, $contents);
	$result = file_get_contents($cachefile);
	return $result;
	
}

// Sets up widget
function lastfmWidget_init()
{
    register_sidebar_widget('Last.fm', 'lastfmWidget');
	register_widget_control('Last.fm', 'lastfmWidgetControl');
	$cacheDir = $_SERVER['DOCUMENT_ROOT'] . "/wp-content/lastfmcache/";
	if (file_exists($cacheDir) === false) {
		mkdir($cacheDir, 0755);
	}
}

// The actual widget
function lastfmWidget($args)
{
    extract($args);
    $opt = get_option('lastfmArtistsOptions');
    echo $before_widget . $before_title . $opt['title'] . $after_title;
	if (is_admin()) {
		echo $after_widget;
	}else{
		if ($result = lastfm_main()) {
			print($result);
		} else {
			echo 'No images to display. Try again later.';
		}
    	echo $after_widget;
	}
}

function lastfmWidgetControl() {
	$data = get_option('lastfmArtistsOptions');
	if (isset($_POST['lastfmArtistsSubmit'])) {
		$data['title'] = strip_tags(stripslashes($_POST['lastfm_title']));
		update_option('lastfmArtistsOptions', $data);
	}
	?>
	<p>
		<label for="lasfm_title">Widget Title: </label>
		<input type="text" name="lastfm_title" value="<?php echo $data['title']; ?>" />
		<input type="hidden" id="lastfmArtistsSubmit" name="lastfmArtistsSubmit" value="1" />
	</p>
	<p>
		For other options, go to the <a href="plugins.php?page=lastfmartists/lastfm.php">Last.fm Artists settings page.</a>
	</p>
	<?php
}

// Sets up submenu admin page
function lastfmAdminPanel()
{
    add_submenu_page('plugins.php', 'Last.fm Artists Plugin', 'Last.fm Artists', 8, __file__, 'lastfm_print');
}

add_action('admin_menu', 'lastfmAdminPanel');
add_action('plugins_loaded', 'lastfmWidget_init');
add_filter('the_content', 'lastfm_filter');

function lastfmartists_print($args = NULL) {
	if (isset($args)) {
		if ($result = lastfm_main($args)) {
			print($result);
		} else {
			echo 'No images to display. Try again later.';
		}
	}else {
		if ($result = lastfm_main()) {
			print($result);
		} else {
			echo 'No images to display. Try again later.';
		}
	}
}

function lastfm_filter($content) {
	while (strpos($content, '{lastfmartists') !== false) {
		if (eregi('\{lastfmartists\|([0-9]+)\|([0-9a-z]+)\|([0-9]+)\}', $content, $out)) {
			$args['number']  = $out[1];
			$args['time'] = $out[2];
			$args['size'] = $out[3];
			
			ob_start();
			lastfmartists_print($args);
			$result = ob_get_contents();
    		ob_clean();
		}elseif (eregi('\{lastfmartists\|([0-9]+)\|([0-9a-z]+)\}', $content, $out)) {
			$args['number']  = $out[1];
			$args['time'] = $out[2];
			
			ob_start();
			lastfmartists_print($args);
			$result = ob_get_contents();
    		ob_clean();
		}elseif (eregi('\{lastfmartists\|([0-9]+)\}', $content, $out)) {
			$args['number']  = $out[1];
			
			ob_start();
			lastfmartists_print($args);
			$result = ob_get_contents();
    		ob_clean();
		}else{
			ob_start();
			lastfmartists_print();
			$result = ob_get_contents();
    		ob_clean();
		}
		
		$content = preg_replace('/{lastfmartists.*}/', $result, $content);
		return $content;
	}
}

function lastfm_print() {
$cacheDir = $_SERVER['DOCUMENT_ROOT'] . "/wp-content/lastfmcache/";
if (file_exists($cacheDir) === false) {
	if (mkdir($cacheDir, 0755) === false) {
		?>
		<div class="error"><p><strong>
			<?php echo "Cache Directory could not be created.  Please create cache directory at '/wp-content/lastfmcache/' with CHMOD of 0755."; ?>
		</strong></p></div>
		<?php
	}
}

if (isset($_POST['updateOptions'])) {
	$url = 'http://www.last.fm/user/' . $_POST['lastfm_user'];
	$headers = get_headers($url);
	if ($_POST['lastfm_user'] == '' || $headers[0] != 'HTTP/1.0 200 OK') {
		$warning = '1';
		?>
		<div class="error"><p><strong><?php echo "Please enter correct username.\n";?></strong></p></div>
		<?php
	}
	if (is_numeric($_POST['lastfm_size']) === false ) {
		$warning = '1';
		?>
		<div class="error"><p><strong><?php echo "Please enter a number for the size.\n";?></strong></p></div>
		<?php
	}
	if (is_numeric($_POST['lastfm_number']) === false ) {
		$warning = '1';
		?>
		<div class="error"><p><strong><?php echo "Please enter a number for the number of artists shown.\n";?></strong></p></div>
		<?php
	}
	if ($_POST['lastfm_number'] > 20 ) {
		$warning = '1';
		?>
		<div class="error"><p><strong><?php echo "Please enter a resonable number for the number of artists shown.\n";?></strong></p></div>
		<?php
	}
	
	if ($warning != '1') {
		//if ($_POST['lastfm_time'] == 'weekly') {
		//	$feed = 'http://ws.audioscrobbler.com/2.0/?method=user.getweeklyartistchart&user=' . $_POST['lastfm_user'] . "&api_key=ffd6c3e445e45d0d4bc124184853b772";
		//} else {
		//	$feed = 'http://ws.audioscrobbler.com/2.0/?method=user.gettopartists&user=' . $_POST['lastfm_user'] . '&api_key=ffd6c3e445e45d0d4bc124184853b772';
		//}
		
		$size = round(abs($_POST['lastfm_size']));
		$number = round(abs($_POST['lastfm_number']));
		$title = strip_tags(stripslashes($_POST['lastfm_title']));
		$data = array('size' => $size, 'number' => $number,
				'user' => $_POST['lastfm_user'], 'time' => $_POST['lastfm_time'], 
				'title' => $_POST['lastfm_title'], 'feed' => $feed);
		
		if (update_option('lastfmArtistsOptions', $data)) {
			?>
			<div class="updated"><p><strong><?php echo "Settings Updated.\n";?></strong></p></div>
			<?php
		} else {
			?>
			<div class="updated"><p><strong><?php echo "No Settings Were Changed.\n";?></strong></p></div>
			<?php
		}
	}
} 
if (isset($_POST['clearCache'])) {
	foreach (glob('../'.$cacheDir.'*') as $filename) {
		unlink($filename);
	}
	?>
		<div class="updated"><p><strong><?php echo "All cached files deleted.\n"; ?></strong></p></div>
	<?php
}
$info = get_option('lastfmArtistsOptions');
?>
<div class="wrap">
	<h2>Last.fm Artists Options</h2>
	<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
		<table class="form-table">
			<tbody>
				<tr valign="top">
					<th scope="row">Title of Widget:</th> 
					<td><input type="text" name="lastfm_title" value="<?php echo ($info['title']); ?>" /></td>
				</tr>
				<tr valign="top">
					<th scope="row">Last.fm Username:</th> 
					<td>
						<input type="text" name="lastfm_user" value="<?php echo ($info['user']); ?>" />
					</td>
				</tr>
				<tr valign="top">
					<th scope="row">Size of the images:</th> 
					<td>
						<input type="text" name="lastfm_size" value="<?php echo ($info['size']); ?>" />px
						<br />
						This sets the width and height of the image.
						<br />
						The image will be square so only one value is needed.
					</td>
				</tr>
				<tr valign="top">
					<th scope="row">Number of artists shown:</th>
					<td>
						<input type="text" name="lastfm_number" value="<?php echo ($info['number']); ?>" />
						<br />
						Values 1 - 20 are accepted.
					</td>
				</tr>
				<tr valign="top">
					<th scope="row">Select your time period</th>
					<td><select name="lastfm_time">
						<?php
							$times = array('Overall', 'Weekly', '3 Month', '6 Month', '12 Month');
							foreach ($times as $time) {
								$tmptime = preg_replace('/\s+/', '', strtolower($time));
								if ($tmptime == $info['time']) {
									?><option value="<?php echo $tmptime; ?>" selected="selected"><?php echo $time; ?></option><?php
								} else {
									?><option value="<?php echo $tmptime; ?>"><?php echo $time; ?></option><?php
								}
							}
						?>
					</select></td>
				</tr>
			</tbody>
		</table>
		<input type="hidden" name="action" value="update" />
		<p class="submit">
			<input type="submit" name="updateOptions" value="<?php echo ("Update Options >>") ?>" />
			<input type="submit" name="clearCache" value="<?php echo ("Clear Cache >>") ?>" />
		</p>
	</form>
</div>
<?php
}
?>
